@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: afinet.c,v 1.9 2001/02/24 12:38:02 bazsi Exp $
+ * $Id: afinet.c,v 1.11 2001/05/04 12:37:43 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -201,6 +201,7 @@ void afinet_src_set_encrypt(struct log_source_driver *c, UINT32 v)
 	werror("Warning: network encryption not implemented yet.\n");
 }
 
+
 static int 
 do_open_afinet_connection(struct afsocket_source *c,
 			  struct io_fd *client,
@@ -362,10 +363,10 @@ struct log_source_driver *make_afinet_source(struct address_info *bind_addr, UIN
      (vars
        (dest object io_fd)
        (dest_buf object abstract_buffer)
+       (sync_freq simple UINT32)
        (conn_fd object connect_fd)
        (cfg object syslog_config)))
 */
-
 /* af_inet destination */
 void afinet_dest_set_auth(struct log_dest_driver *c, UINT32 v)
 {	
@@ -408,6 +409,12 @@ void afinet_dest_set_destport(struct log_dest_driver *c, UINT32 port,
 	inet_address_setport(&dest->dest_addr, port, service, proto);
 }
 
+void afinet_dest_set_syncfreq(struct log_dest_driver *c, UINT32 syncfreq)
+{
+ 	CAST_SUBTYPE(afinet_dest, dest, c);
+	dest->sync_freq = syncfreq;
+}
+
 static int do_init_afinet_dest(struct log_handler *c, 
 			       struct syslog_config *cfg,
 			       struct persistent_config *persistent)
@@ -429,8 +436,11 @@ static int do_init_afinet_dest(struct log_handler *c,
 	if (fd < 0) {
 		return ST_FAIL | ST_QUIT;
 	}
-	if (!self->dest_buf)
-		self->dest_buf = make_pkt_buffer(self->super.super.log_fifo_size);
+	if (!self->dest_buf) {
+		self->dest_buf = make_pkt_buffer_ext(self->sync_freq, 
+                				     MAX(self->super.super.log_fifo_size + 1, 
+                                                         self->sync_freq));
+        }
 
 	self->conn_fd = io_connect(cfg->backend, 
 		 	           fd, 
@@ -452,6 +462,7 @@ do_afinet_dest_connected(struct afsocket_dest *c,
 {
 	CAST(afinet_dest, self, c);
 	struct resource_node *n;
+	CAST(inet_address_info, inet_addr, self->super.dest_addr);
 
 	if (!self->cfg->living) {
 		werror("Connection established for an old configuration, dropping...\n");
@@ -469,12 +480,12 @@ do_afinet_dest_connected(struct afsocket_dest *c,
 				      make_driver_reinit_on_close
 				      (&self->super.super.super, n, 
 				       self->cfg->time_reopen, 
-				       self->cfg));
+				       self->cfg,
+				       c_format("AF_INET(%S:%i)", inet_addr->ip, inet_addr->port)));
 	}
 	else {
-		CAST(inet_address_info, inet_addr, self->super.dest_addr);
 
-		werror("Error connecting to remote host (%S), reattempting in %i seconds\n", inet_addr->ip, self->cfg->time_reopen);
+		werror("Error connecting to remote host AF_INET(%S:%i), reattempting in %i seconds\n", inet_addr->ip, inet_addr->port, self->cfg->time_reopen);
 		self->dest = NULL;
 		io_callout(self->cfg->backend, 
 			   self->cfg->time_reopen, 
