@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: center.c,v 1.16 2001/02/24 12:38:02 bazsi Exp $
+ * $Id: center.c,v 1.17 2001/05/04 12:37:43 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -160,6 +160,7 @@ make_driver_reinit(struct log_handler *handler, struct syslog_config *cfg)
          (handler object log_handler)
 	 (res simple "struct resource_node *")
 	 (cfg object syslog_config)
+	 (desc string)
          (time_reopen simple int)))
 */
 
@@ -169,7 +170,7 @@ static int do_driver_on_close(struct close_callback *c, int reason)
 
 	/* schedule a reopen */
 	if (self->time_reopen)
-		werror("Connection broken, reopening in %i seconds\n", self->time_reopen);
+		werror("Connection broken to %S, reopening in %i seconds\n", self->desc, self->time_reopen);
 	if (self->res)
 		KILL_RESOURCE_NODE(self->cfg->resources, self->res);
 	io_callout(self->cfg->backend, self->time_reopen, 
@@ -182,7 +183,8 @@ struct close_callback *
 make_driver_reinit_on_close(struct log_handler *handler, 
 			    struct resource_node *res, 
 			    int time_reopen, 
-			    struct syslog_config *cfg)
+			    struct syslog_config *cfg,
+			    struct ol_string *desc)
 {
 	NEW(driver_reinit_on_close, self);
 
@@ -191,6 +193,7 @@ make_driver_reinit_on_close(struct log_handler *handler,
 	self->res = res;
 	self->time_reopen = time_reopen;
 	self->cfg = cfg;
+	self->desc = desc;
 	return &self->super;
 }
 
