@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: center.h,v 1.8 2001/02/24 12:38:03 bazsi Exp $
+ * $Id: center.h,v 1.9 2001/05/04 12:37:43 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -72,7 +72,8 @@ struct close_callback *
 make_driver_reinit_on_close(struct log_handler *handler, 
 			    struct resource_node *res, 
 			    int time_reopen, 
-			    struct syslog_config *cfg);
+			    struct syslog_config *cfg,
+			    struct ol_string *desc);
 
 #endif
 
