@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: log.c,v 1.21 2001/02/24 12:38:03 bazsi Exp $
+ * $Id: log.c,v 1.22 2001/05/04 12:37:43 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -38,6 +38,9 @@
 #include "log.h.x"
 #undef CLASS_DEFINE
 
+static char aix_fwd_string[] = "Message forwarded from ";
+static char repeat_msg_string[] = "last message repeated";
+
 static void parse_log_msg(struct log_info *lm, UINT32 length, UINT8 *data)
 {
 	char *src;
@@ -72,21 +75,32 @@ static void parse_log_msg(struct log_info *lm, UINT32 length, UINT8 *data)
 			left--;
 		}
 	}
+	/* No priority info in the buffer? Just assign a default. */
 	else {
-		lm->pri = LOG_USER | LOG_NOTICE;
+	        lm->pri = LOG_USER | LOG_NOTICE;
 	}
-	while (left && *src == ' ') {
+
+
+	while (left && *src == ' ') {  /* Move past whitespace */
 		src++;
 		left--;
 	}
-	if (left >= 15) {
-		if (src[3] == ' ' && src[6] == ' ' && src[9] == ':' && src[12] == ':') {
-			struct tm tm, *nowtm;
 
+	/* If the next chars look like a date, then read them as a date. */
+	if (left >= 15) {
+  	        /* Expected buffer format: MMM DD HH:MM:SS ... */
+
+	        if (src[3] == ' ' && src[6] == ' ' &&
+		    src[9] == ':' && src[12] == ':') {
+	                struct tm tm, *nowtm;
+
+			/* Just read the buffer data into a textual
+			   datestamp. */
 			lm->date = c_format_cstring("%s", 15, src);
 			src += 15;
 			left -= 15;
 
+			/* And also make struct time timestamp for the msg */
 			nowtm = localtime(&now);
 			memset(&tm, 0, sizeof(tm));
 			strptime(lm->date->data, "%b %e %H:%M:%S", &tm);
@@ -97,54 +111,115 @@ static void parse_log_msg(struct log_info *lm, UINT32 length, UINT8 *data)
 			lm->stamp = mktime(&tm);
 		}
 	}
-
-	if (lm->date) {
+ 
+        if (lm->date) {
+	        /* Expected format: hostname program[pid]: */
+	        /* Possibly: Message forwarded from hostname: ... */
+	        char *hostname_start = NULL;
+		int hostname_len = 0;
+		
 		while (left && *src == ' ') {
 			src++; /* skip whitespace */
 			left--;
 		}
-		oldsrc = src;	
-		oldleft = left;
-		while (left && *src != ' ' && *src != ':' && *src != '[') {
-			src++;
-			left--;
+
+		/* Detect funny AIX syslogd forwarded message. */
+		if (left >= (sizeof(aix_fwd_string) - 1) &&
+		    !memcmp(src, aix_fwd_string, sizeof(aix_fwd_string) - 1)) {
+
+		        oldsrc = src;
+			oldleft = left;
+			src += sizeof(aix_fwd_string) - 1;
+			left -= sizeof(aix_fwd_string) - 1;
+			hostname_start = src;
+			hostname_len   = 0;
+			while (left && *src != ':') {
+				src++;
+				left--;
+				hostname_len++;
+			}
+			while (left && (*src == ' ' || *src == ':')) {
+				src++; /* skip whitespace */
+				left--;
+			}
 		}
-		if (left && *src == ' ') {
-			/* this was a hostname */
-			lm->host = c_format_cstring("%s", oldleft - left, oldsrc);
+
+		/* Now, try to tell if it's a "last message repeated" line */
+		if (left >= sizeof(repeat_msg_string) &&
+		    !memcmp(src, repeat_msg_string, 
+			    sizeof(repeat_msg_string) - 1)) {
+			; /* It is. Do nothing since there's no hostname or
+			     program name coming. */
 		}
+		/* It's a regular ol' message. */
 		else {
+			/* If we haven't already found the original hostname,
+			   look for it now. */
+
+		        oldsrc = src;	
+			oldleft = left;
+
+			while (left && *src != ' ' && *src != ':' 
+			       && *src != '[') {
+		 		src++;
+				left--;
+			}
+
+			if (left && *src == ' ') {
+		  		/* This was a hostname. It came from a
+				   syslog-ng, since syslogd doesn't send
+				   hostnames. It's even better then the one
+				   we got from the AIX fwd message, if we
+				   did. */
+				hostname_start = oldsrc;
+				hostname_len   = oldleft - left;
+			} else {
+		  		src = oldsrc;
+		  		left = oldleft;
+			}
+
+			/* Skip whitespace. */
+			while (left && *src == ' ') {
+				src++;
+				left--;
+			}
+
+			/* Try to extract a program name */
+			oldsrc = src;
+			oldleft = left;
+			while (left && *src != ':' && *src != '[') {
+				src++;
+				left--;
+			}
+			if (left) {
+				lm->program = c_format_cstring("%s",
+						oldleft - left, oldsrc);
+			} 
+			
 			src = oldsrc;
 			left = oldleft;
 		}
 
-		while (left && *src == ' ') {
-			src++; /* skip whitespace */
-			left--;
-		}
-		
-		oldsrc = src;
-		oldleft = left;
-
-		while (left && *src != ':' && *src != '[') {
-			src++;
-			left--;
-		}
-		if (left) {
-			lm->program = c_format_cstring("%s", oldleft - left, oldsrc);
-		}
-		src = oldsrc;
-		left = oldleft;
+		/* If we did manage to find a hostname, store it. */
+		if (hostname_start)
+			lm->host = c_format_cstring("%s", hostname_len,
+						    hostname_start);
 	}
 	else {
+	        /* Different format */
+
 		oldsrc = src;
 		oldleft = left;
+		/* A kernel message? Use 'kernel' as the program name. */
 		if ((lm->pri & LOG_FACMASK) == LOG_KERN) {
 			lm->program = c_format_cstring("kernel");
 		}
+		/* No, not a kernel message. */
 		else {
-			while (left && *src != ' ' && *src != '[' && *src != ':' && 
-			       *src != '/' && *src != ',' && *src != '<') {
+		        /* Capture the program name */ 
+		        while (left && *src != ' ' && *src != '['
+			       && *src != ':' && *src != '/' &&
+			       *src != ',' && *src != '<') {
 				src++;
 				left--;
 			}
