@@ -19,7 +19,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: main.c,v 1.29 2001/02/24 12:38:03 bazsi Exp $
+ * $Id: main.c,v 1.31 2001/05/04 12:37:43 bazsi Exp $
  *
  ***************************************************************************/
 
@@ -38,6 +38,8 @@
 #include <errno.h>
 #include <string.h>
 #include <getopt.h>
+#include <grp.h>
+#include <pwd.h>
 
 #include "main.c.x"
 
@@ -181,31 +183,99 @@ int main_loop(struct syslog_backend *backend)
 	return 0;
 }
 
-void go_background()
+int go_background()
 {
 	pid_t pid;
+	int wakeup_pipe[2];
+	char buf[1];
 	
+	if (pipe(wakeup_pipe) < 0) {
+		werror("Error in pipe(), (%z)\n", strerror(errno));
+		return -1;
+	}
 	pid = fork();
 	if (pid == 0) {
-		int fd = open(pidfilename, O_CREAT | O_WRONLY | O_NOCTTY | O_TRUNC, 0600);
+		int fd;
+		
+		close(wakeup_pipe[0]);
+		fd = open(pidfilename, O_CREAT | O_WRONLY | O_NOCTTY | O_TRUNC, 0600);
 		if (fd != -1) {
 			struct ol_string *pid_s = c_format("%i\n", getpid());
 			write(fd, pid_s->data, pid_s->length);
 			ol_string_free(pid_s);
 			close(fd);
 		}
-		return;
+		return wakeup_pipe[1];
 	}
 	else if (pid == -1) {
 		werror("Cannot fork(), (%z)\n", strerror(errno));
 		exit(1);
 	}
-	signal(SIGTERM, sig_term);
-	while (sigtermrecvd == 0)
-		pause();
+	close(wakeup_pipe[1]);
+	if (read(wakeup_pipe[0], buf, sizeof(buf)) == 1 && buf[0] == 0) {
+		exit(0);
+	}
+	else {
+		exit(1);
+	}
 	exit(0);
 }
 
+int resolve_user_group(char *arg, uid_t *uid, gid_t *gid)
+{
+	char *user, *group;
+	struct passwd *pw;
+	struct group *gr;
+	
+	*uid = 0;
+	user = strtok(arg, ":.");
+	group = strtok(NULL, "");
+	if (user) {
+		pw = getpwnam(user);
+		if (!pw) {
+			*uid = pw->pw_uid;
+		}
+		else {
+			*uid = atoi(user);
+		}
+		if (*uid == 0)
+			return 0;
+	}
+	else {
+		return 0;
+	}
+	if (group && gid) {
+		*gid = 0;
+		gr = getgrnam(group);
+		if (!gr) {
+			*gid = gr->gr_gid;
+		}
+		else {
+			*gid = atoi(group); 
+		}
+		if (*gid == 0)
+			return 0;
+	}
+	return 1;
+}
+
+int resolve_group(char *group, gid_t *gid)
+{
+	struct group *gr;
+
+	*gid = 0;
+	gr = getgrnam(group);
+	if (!gr) {
+		*gid = gr->gr_gid;
+	}
+	else {
+		*gid = atoi(group); 
+	}
+	if (*gid == 0)
+		return 0;
+	return 1;
+}
+
 void usage(void)
 {
 	werror("Usage: syslog-ng [options]\n"
@@ -216,6 +286,9 @@ void usage(void)
 	       "  -f <fname>, --cfgfile=<fname>    Set config file name, default=" PATH_SYSLOG_NG_CONF "\n"
 	       "  -V, --version                    Display version number (" PACKAGE " " VERSION ")\n"
 	       "  -p <fname>, --pidfile=<fname>    Set pid file name, default=" PATH_PIDFILE "\n"
+               "  -C <dir>, --chroot=<dir>         Chroot to directory\n"
+               "  -u <user>, --user=<user>         Switch to user\n"
+               "  -g <group>, --group=<group>      Switch to group\n"
 #ifdef YYDEBUG
 	       "  -y, --yydebug                    Turn on yacc debug messages\n"
 #endif
@@ -232,6 +305,9 @@ int main(int argc, char **argv)
 {
 	int do_fork = 1;
 	int opt;
+	int wakeup_fd = -1;
+	char *chroot_dir = NULL;
+	int uid = 0, gid = 0;
 
 	NEW(syslog_backend, backend);
 	struct option syslog_ng_options[] = {
@@ -241,6 +317,9 @@ int main(int argc, char **argv)
 		{ "verbose", no_argument, NULL, 'v' },
 		{ "help", no_argument, NULL, 'h' },
 		{ "version", no_argument, NULL, 'V' },
+                { "chroot", required_argument, NULL, 'C' },
+                { "user", required_argument, NULL, 'u' },
+                { "group", required_argument, NULL, 'g' },
 #ifdef YYDEBUG
 		{ "yydebug", no_argument, NULL, 'y' },
 #endif
@@ -252,7 +331,7 @@ int main(int argc, char **argv)
 	gc_idle_threshold = 100;
 	gc_busy_threshold = 3000;
 
-	while ((opt = getopt_long(argc, argv, "f:p:dvhyV", syslog_ng_options, NULL)) != -1) {
+	while ((opt = getopt_long(argc, argv, "f:p:dvhyVC:u:g:", syslog_ng_options, NULL)) != -1) {
 		switch (opt) {
 		case 'f':
 			strncpy(cfgfilename, optarg, sizeof(cfgfilename));
@@ -269,6 +348,17 @@ int main(int argc, char **argv)
 		case 'V':
 			werror(PACKAGE " " VERSION "\n");
 			return 0;
+		case 'C':
+			chroot_dir = optarg;
+			break;
+		case 'u':
+			if (!resolve_user_group(optarg, &uid, gid ? NULL : &gid))
+				usage();
+			break;
+		case 'g':
+			if (!resolve_group(optarg, &gid))
+				usage();
+			break;
 #ifdef YYDEBUG
 		case 'y':
 			yydebug = 1;
@@ -292,21 +382,44 @@ int main(int argc, char **argv)
 		do_fork = 0;
 	}
 	if (do_fork) {
-		go_background();
+		wakeup_fd = go_background();
 	}
 	if (!CONFIG_INIT(backend->configuration, NULL)) {
+		char res = 1;
+		
 		werror("Error initializing configuration, exiting.\n");
-		if (do_fork) kill(getppid(), SIGTERM);
+		if (wakeup_fd != -1) {
+			write(wakeup_fd, &res, 1);
+			close(wakeup_fd);
+		}
 		return 2;
 	}
+	if (chroot_dir) {
+		if (chroot(chroot_dir) < 0) {
+			werror("Error chrooting, exiting.\n");
+			return 3;
+		}
+	}
+	
+	if (uid || gid) {
+		setgid(gid);
+		setgroups(0, NULL);
+		setuid(uid);
+	}
 
 	if (do_fork) {
+
 		set_error_internal();
 		close(0);
 		close(1);
 		close(2);
 		setsid();
-		kill(getppid(), SIGTERM); 
+	}
+	if (wakeup_fd != -1) {
+		char res = 0;
+
+		write(wakeup_fd, &res, 1);
+		close(wakeup_fd);
 	}
 	notice("syslog-ng version %z starting\n", VERSION);
 	return main_loop(backend);
